@@ -21,7 +21,6 @@ use crate::wchar::{encode_byte_to_char, prelude::*};
 use crate::wutil::encoding::{mbrtowc, mbstate_t, zero_mbstate};
 use crate::wutil::fish_wcstol;
 use std::collections::VecDeque;
-use std::ops::ControlFlow;
 use std::os::fd::RawFd;
 use std::os::unix::ffi::OsStrExt;
 use std::ptr;
@@ -563,6 +562,7 @@ fn parse_mask(mask: u32) -> Modifiers {
         ctrl: (mask & 4) != 0,
         alt: (mask & 2) != 0,
         shift: (mask & 1) != 0,
+        sup: (mask & 8) != 0,
     }
 }
 
@@ -701,21 +701,25 @@ pub trait InputEventQueuer {
                                 _ => 0,
                             });
                         }
-                        match self.parse_codepoint(
-                            &mut state,
-                            &mut key,
+                        match decode_input_byte(
                             &mut seq,
-                            &buffer,
-                            i,
+                            &mut state,
+                            &buffer[..i + 1],
                             &mut consumed,
-                            &mut have_escape_prefix,
                         ) {
-                            ControlFlow::Continue(codepoint_complete) => {
-                                if codepoint_complete && i + 1 == buffer.len() {
+                            DecodeState::Incomplete => (),
+                            DecodeState::Complete => {
+                                if have_escape_prefix && i != 0 {
+                                    have_escape_prefix = false;
+                                    let c = seq.as_char_slice().last().unwrap();
+                                    key = Some(Key::from(alt(*c)));
+                                }
+                                if i + 1 == buffer.len() {
                                     break true;
                                 }
                             }
-                            ControlFlow::Break(()) => {
+                            DecodeState::Error => {
+                                self.push_front(CharEvent::from_check_exit());
                                 break false;
                             }
                         }
@@ -789,73 +793,6 @@ pub trait InputEventQueuer {
         }
     }
 
-    fn parse_codepoint(
-        &mut self,
-        state: &mut mbstate_t,
-        out_key: &mut Option<Key>,
-        out_seq: &mut WString,
-        buffer: &[u8],
-        i: usize,
-        consumed: &mut usize,
-        have_escape_prefix: &mut bool,
-    ) -> ControlFlow<(), bool> {
-        let mut res: char = '\0';
-        let read_byte = buffer[i];
-        if crate::libc::MB_CUR_MAX() == 1 {
-            // single-byte locale, all values are legal
-            // FIXME: this looks wrong, this falsely assumes that
-            // the single-byte locale is compatible with Unicode upper-ASCII.
-            res = read_byte.into();
-            out_seq.push(res);
-            return ControlFlow::Continue(true);
-        }
-        let mut codepoint = u32::from(res);
-        let sz = unsafe {
-            mbrtowc(
-                std::ptr::addr_of_mut!(codepoint).cast(),
-                std::ptr::addr_of!(read_byte).cast(),
-                1,
-                state,
-            )
-        } as isize;
-        match sz {
-            -1 => {
-                FLOG!(reader, "Illegal input");
-                *consumed += 1;
-                self.push_front(CharEvent::from_check_exit());
-                return ControlFlow::Break(());
-            }
-            -2 => {
-                // Sequence not yet complete.
-                return ControlFlow::Continue(false);
-            }
-            0 => {
-                // Actual nul char.
-                *consumed += 1;
-                out_seq.push('\0');
-                return ControlFlow::Continue(true);
-            }
-            _ => (),
-        }
-        if let Some(res) = char::from_u32(codepoint) {
-            // Sequence complete.
-            if !fish_reserved_codepoint(res) {
-                if *have_escape_prefix && i != 0 {
-                    *have_escape_prefix = false;
-                    *out_key = Some(alt(res));
-                }
-                *consumed += 1;
-                out_seq.push(res);
-                return ControlFlow::Continue(true);
-            }
-        }
-        for &b in &buffer[*consumed..i] {
-            out_seq.push(encode_byte_to_char(b));
-            *consumed += 1;
-        }
-        ControlFlow::Continue(true)
-    }
-
     fn parse_csi(&mut self, buffer: &mut Vec<u8>) -> Option<Key> {
         let mut next_char = |zelf: &mut Self| zelf.try_readb(buffer).unwrap_or(0xff);
         // The maximum number of CSI parameters is defined by NPAR, nominally 16.
@@ -1325,6 +1262,65 @@ pub trait InputEventQueuer {
     }
 }
 
+pub(crate) enum DecodeState {
+    Incomplete,
+    Complete,
+    Error,
+}
+
+pub(crate) fn decode_input_byte(
+    out_seq: &mut WString,
+    state: &mut mbstate_t,
+    buffer: &[u8],
+    consumed: &mut usize,
+) -> DecodeState {
+    use DecodeState::*;
+    let mut res: char = '\0';
+    let read_byte = *buffer.last().unwrap();
+    if crate::libc::MB_CUR_MAX() == 1 {
+        // single-byte locale, all values are legal
+        // FIXME: this looks wrong, this falsely assumes that
+        // the single-byte locale is compatible with Unicode upper-ASCII.
+        res = read_byte.into();
+        out_seq.push(res);
+        return Complete;
+    }
+    let mut codepoint = u32::from(res);
+    match unsafe {
+        mbrtowc(
+            std::ptr::addr_of_mut!(codepoint).cast(),
+            std::ptr::addr_of!(read_byte).cast(),
+            1,
+            state,
+        )
+    } as isize
+    {
+        -1 => {
+            FLOG!(reader, "Illegal input");
+            *consumed += 1;
+            return Error;
+        }
+        -2 => {
+            // Sequence not yet complete.
+            return Incomplete;
+        }
+        _ => (),
+    }
+    if let Some(res) = char::from_u32(codepoint) {
+        // Sequence complete.
+        if !fish_reserved_codepoint(res) {
+            *consumed += 1;
+            out_seq.push(res);
+            return Complete;
+        }
+    }
+    for &b in &buffer[*consumed..] {
+        out_seq.push(encode_byte_to_char(b));
+        *consumed += 1;
+    }
+    Complete
+}
+
 /// A simple, concrete implementation of InputEventQueuer.
 pub struct InputEventQueue {
     data: InputData,
