@@ -183,7 +183,7 @@ fn maybe_issue_path_warning(
             )
         );
     }
-    printf!("\n");
+    eprintf!("\n");
 }
 
 /// Finds the path of an executable named `cmd`, by looking in $PATH taken from `vars`.
@@ -707,7 +707,7 @@ fn path_remoteness(path: &wstr) -> DirRemoteness {
         // these are in varying headers. Simply hard code them.
         // Note that we treat FUSE filesystems as remote, which means we lock less on such filesystems.
         // NOTE: The cast is necessary for 32-bit systems because of the 4-byte CIFS_MAGIC_NUMBER
-        match usize::try_from(buf.f_type).unwrap() {
+        match buf.f_type as usize  {
             0x5346414F | // AFS_SUPER_MAGIC - Andrew File System
             0x6B414653 | // AFS_FS_MAGIC - Kernel AFS and AuriStorFS
             0x73757245 | // CODA_SUPER_MAGIC - Coda File System
